package question1;


import java.io.Serializable;
import java.io.ObjectOutputStream;
import java.io.ObjectInputStream;
import java.io.FileOutputStream;
import java.io.FileInputStream;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.TreeSet;
import java.util.Collections;

public class Diviseur implements DiviseurI{
  private Message[] messages;
  private Map<Integer,Boolean> messagesLus;
  private String nom;
  
  public Diviseur(String nom){
    this.nom = nom;
    try{
      restoreMessages(this, nom);
    }catch(Exception e){
      this.messages = null;
      this.messagesLus = new TreeMap<Integer,Boolean>();
    }
  }
  
  public synchronized void envoyer(Message... messages) throws Exception{
    if(!tousLesMessagesSontLus()) throw new Exception("tous les messages n'ont pas t lus ...");
    this.messages = messages;
    messagesLus.clear();
    for(int i=0;i<messages.length;i++)
      messagesLus.put(i,false);
    saveMessages(this,nom);
  }
  
  private boolean tousLesMessagesSontLus(){
    for(Integer i : messagesLus.keySet()){
      if(!messagesLus.get(i)) return false;
    }
    return true;
  }
    
  public synchronized Message lire(int index) throws Exception{
    try{
      Message m = messages[index];
      messagesLus.put(index,true);
      saveMessages(this,nom);
      return m;
    }catch(ArrayIndexOutOfBoundsException e){
      throw new Exception("cet index est erron ...");
    }catch(NullPointerException e){
      throw new Exception("pas de messages  consulter ...");
    }
  }
  
  public synchronized void vider() throws Exception{
    boolean tousLus = tousLesMessagesSontLus();
    this.messages = null;
    this.messagesLus.clear();
    saveMessages(this,nom);
    if(!tousLus) throw new Exception("tous les messages n'ont pas t lus ...");
  }

  public synchronized Set<Integer> messagesNonLus() throws Exception{
    if(this.messages==null) throw new Exception("aucun envoi n'a eu lieu ...");
    Set<Integer> set = new TreeSet<Integer>();
    for(Integer i : messagesLus.keySet()){
      if(!messagesLus.get(i)) set.add(i);
    }
    return set;
  }
  


  private final static String FILENAME1 = "splitterMessages_";
  private final static String FILENAME2 = "splitterMessagesLus_";
  
  private static void saveMessages(Diviseur splitter, String nom) throws Exception{
    ObjectOutputStream oos = new ObjectOutputStream( new FileOutputStream(FILENAME1+nom+".ser"));
    oos.writeObject(splitter.messages);
    oos.close();
    oos = new ObjectOutputStream( new FileOutputStream(FILENAME2+nom+".ser"));
    oos.writeObject(splitter.messagesLus);
    oos.close();
  }
  
  private static void restoreMessages(Diviseur splitter, String nom) throws Exception{
    ObjectInputStream ois = new ObjectInputStream( new FileInputStream(FILENAME1+nom+".ser"));
    splitter.messages = (Message[])ois.readObject();
    ois.close();
    ois = new ObjectInputStream( new FileInputStream(FILENAME2+nom+".ser"));
    splitter.messagesLus = (Map<Integer,Boolean>)ois.readObject();
    ois.close();
  }
  
//   public Set<Integer> readMessages() throws Exception{
//     if(this.messages==null) throw new Exception("aucun envoi n'a eu lieu ...");
//     Set<Integer> set = new TreeSet<Integer>();
//     for(Integer i : readMessages.keySet()){
//       if(readMessages.get(i)) set.add(i);
//     }
//     return set;
//   }

//   public int size() throws Exception{
//     if(this.messages==null || allReadMessages()) throw new Exception("tous les messages ont t lus ...");
//     return messages.length;
//   }

// import java.io.ObjectOutputStream;
// import java.io.ObjectInputStream;
// import java.io.FileOutputStream;
// import java.io.FileInputStream;

//   private final static String FILENAME1 = "splitterMessages.ser";
//   private final static String FILENAME2 = "splitterReadMessages.ser";
//   
//   private void saveMessages() throws Exception{
//     ObjectOutputStream oos = new ObjectOutputStream( new FileOutputStream(FILENAME1));
//     oos.writeObject(messages);
//     oos.close();
//     oos = new ObjectOutputStream( new FileOutputStream(FILENAME2));
//     oos.writeObject(readMessages);
//     oos.close();
//   }
//   
//   private void restoreMessages() throws Exception{
//     ObjectInputStream ois = new ObjectInputStream( new FileInputStream(FILENAME1));
//     messages = (M[])ois.readObject();
//     ois.close();
//     ois = new ObjectInputStream( new FileInputStream(FILENAME2));
//     readMessages = (Map<Integer,Boolean>)ois.readObject();
//     ois.close();
//   }
  
}
